__all__ = [
    '__title__',
    '__summary__',
    '__uri__',
    '__version__',
    '__author__',
    '__email__',
    '__license__',
    '__copyright__',
]

__title__ = 'xcsp-launcher'
__summary__ = 'A unified launcher for XCSP solveur'
__keywords__ = ''
__uri__ = 'https://github.com/crillab/xcsplauncher'
__version__ = '0.7.0'
__author__ = 'Thibault Falque, Romain Wallon'
__email__ = "wallon@cril.fr"
__license__ = 'LGPLv3+'
__copyright__ = '2019-2025 - Univ Artois & CNRS, University of Luxembourg'
